@@ -10,7 +10,7 @@ __author__ = "Shyue Ping Ong, Michael Kocher"
 __copyright__ = "Copyright 2011, The Materials Project"
 __version__ = "1.0"
 __maintainer__ = "Shyue Ping Ong"
-__email__ = "shyue@mit.edu"
+__email__ = "shyuep@gmail.com"
 __status__ = "Production"
 __date__ = "Sep 23, 2011"
 
@@ -313,21 +313,21 @@ class Lattice(MSONable):
     @property
     def alpha(self):
         """
-        Angle alpha of lattice.
+        Angle alpha of lattice in degrees.
         """
         return self._angles[0]
 
     @property
     def beta(self):
         """
-        Angle beta of lattice.
+        Angle beta of lattice in degrees.
         """
         return self._angles[1]
 
     @property
     def gamma(self):
         """
-        Angle gamma of lattice.
+        Angle gamma of lattice in degrees.
         """
         return self._angles[2]
 
@@ -349,11 +349,16 @@ class Lattice(MSONable):
     @property
     def reciprocal_lattice(self):
         """
-        Return the reciprocal lattice.
+        Return the reciprocal lattice. The property is lazily generated for efficiency.
+        This implies that Lattice is an immutable object.
         """
-        v = [np.cross(self._matrix[(i + 1) % 3], self._matrix[(i + 2) % 3])
-             for i in xrange(3)]
-        return Lattice(np.array(v) * 2 * np.pi / self.volume)
+        try:
+            return self._reciprocal_lattice
+        except AttributeError:
+            v = [np.cross(self._matrix[(i + 1) % 3], self._matrix[(i + 2) % 3])
+                for i in xrange(3)]
+            self._reciprocal_lattice = Lattice(np.array(v) * 2 * np.pi / self.volume)
+            return self._reciprocal_lattice
 
     def __repr__(self):
         f = lambda x: "%0.6f" % x
@@ -715,8 +720,8 @@ class Lattice(MSONable):
     def scale(self, new_volume):
         """
         Return a new Lattice with volume new_volume by performing a
-        scaling of the lattice vectors so that length proportions and angles are
-        preserved.
+        scaling of the lattice vectors so that length proportions and angles
+        are preserved.
 
         Args:
             new_volume:
@@ -773,3 +778,52 @@ class Lattice(MSONable):
             represent a square facet.
         """
         return self.reciprocal_lattice.get_wigner_seitz_cell()
+
+    def dot(self, coords_a, coords_b, frac_coords=False):
+        """
+        Compute the scalar product of vector(s).
+
+        Args:
+            coords_a, coords_b:
+                Array-like objects with the coordinates.
+            frac_coords:
+                Boolean stating whether the vector corresponds to fractional or
+                cartesian coordinates.
+
+        Returns:
+            one-dimensional `numpy` array.
+        """
+        coords_a, coords_b = np.reshape(coords_a, (-1,3)), \
+                             np.reshape(coords_b, (-1,3))
+
+        if len(coords_a) != len(coords_b):
+            raise ValueError("")
+
+        if np.iscomplexobj(coords_a) or np.iscomplexobj(coords_b):
+            raise TypeError("Complex array!")
+
+        if not frac_coords:
+            cart_a, cart_b = coords_a, coords_b
+        else:
+            cart_a = np.reshape([self.get_cartesian_coords(vec)
+                                 for vec in coords_a], (-1,3))
+            cart_b = np.reshape([self.get_cartesian_coords(vec)
+                                 for vec in coords_b], (-1,3))
+
+        return np.array([np.dot(a,b) for a,b in zip(cart_a, cart_b)])
+
+    def norm(self, coords, frac_coords=True):
+        """
+        Compute the norm of vector(s).
+
+        Args:
+            coords:
+                Array-like object with the coordinates.
+            frac_coords:
+                Boolean stating whether the vector corresponds to fractional or
+                cartesian coordinates.
+
+        Returns:
+            one-dimensional `numpy` array.
+        """
+        return np.sqrt(self.dot(coords, coords, frac_coords=frac_coords))
