@@ -3,4 +3,5 @@ from .eos import *
 from .pseudos import *
 from .netcdf import *
 from .events import *
-#from .workflow import *
+from .task import *
+from .workflow import *
