@@ -16,6 +16,7 @@ import numpy as np
 
 from pymatgen.core.periodic_table import PeriodicTable
 from pymatgen.util.num_utils import iterator_from_slice
+from pymatgen.util.string_utils import list_strings, is_string
 
 __all__ = [
     "Pseudo",
@@ -493,7 +494,7 @@ def _dict_from_lines(lines, key_nums, sep=None):
 
     Return dict{key1 : value1, key2 : value2, ...}
     """
-    if isinstance(lines, basestring):
+    if is_string(lines):
         lines = [lines]
 
     if not isinstance(key_nums, collections.Iterable):
@@ -955,6 +956,7 @@ class PseudoParser(object):
 
         try:
             header = parsers[ppdesc.name](path, ppdesc)
+
         except Exception as exc:
             raise self.Error(filename + ": " + str(exc))
 
@@ -1013,10 +1015,12 @@ class PseudoTable(collections.Sequence):
         # Store pseudos in a default dictionary with z as key.
         # Note that we can have more than one pseudo for given z.
         # hence the values are lists of pseudos.
-
         if not isinstance(pseudos, collections.Iterable):
             pseudos = [pseudos]
 
+        if is_string(pseudos[0]):
+            pseudos = list_strings(pseudos)
+
         self._pseudos_with_z = collections.defaultdict(list)
 
         for pseudo in pseudos:
