@@ -19,6 +19,7 @@ import pymatgen.core.physical_constants as const
 from pymatgen.serializers.json_coders import MSONable, json_pretty_dump
 from pymatgen.io.smartio import read_structure
 from pymatgen.util.num_utils import iterator_from_slice, chunks
+from pymatgen.util.string_utils import list_strings
 from pymatgen.io.abinitio.task import task_factory, Task, AbinitTask
 from pymatgen.io.abinitio.strategies import Strategy
 
@@ -27,7 +28,6 @@ from .netcdf import ETSF_Reader
 from .abiobjects import Smearing, AbiStructure, KSampling, Electrons
 from .pseudos import Pseudo
 from .strategies import ScfStrategy
-from .task import RunMode
 from .eos import EOS
 
 #import logging
@@ -114,8 +114,7 @@ class WorkLink(object):
         self._products = []
 
         if exts is not None:
-            if isinstance(exts, str):
-                exts = [exts,]
+            exts = list_strings(exts)
 
             for ext in exts:
                 prod = Product(ext, task.odata_path_from_ext(ext))
@@ -165,7 +164,7 @@ class BaseWorkflow(object):
     Error = WorkflowError
 
     # Basename of the pickle database.
-    _PICKLE_FNAME = "__workflow__.pickle"
+    PICKLE_FNAME = "__workflow__.pickle"
 
     # interface modeled after subprocess.Popen
     @abc.abstractproperty
@@ -251,11 +250,16 @@ class BaseWorkflow(object):
         """
         return WorkFlowResults(task_results={task.name: task.results for task in self})
 
-    def pickle_dump(self, protocol=0):
-        """Save the status of the object in pickle format."""
-        filepath = os.path.join(self.workdir, self._PICKLE_FNAME)
+    def build_and_pickle_dump(self, protocol=-1):
+        self.build()
+        self.pickle_dump(protocol=protocol)
 
-        with open(filepath, "w") as fh:
+    def pickle_dump(self, protocol=-1):
+        """Save the status of the object in pickle format."""
+        filepath = os.path.join(self.workdir, self.PICKLE_FNAME)
+
+        mode = "w" if protocol == 0 else "wb" 
+        with open(filepath, mode) as fh:
             pickle.dump(self, fh, protocol=protocol)
 
     @classmethod
@@ -268,13 +272,12 @@ class BaseWorkflow(object):
                 filepath or directory name.
         """
         if os.path.isdir(path):
-            path = os.path.join(path, cls._PICKLE_FNAME)
+            path = os.path.join(path, cls.PICKLE_FNAME)
 
         # TODO atomic transaction.
         with open(path, "rb") as fh:
             new = pickle.load(fh)
 
-        #new.__class__ = cls
         return new
 
 
@@ -335,17 +338,17 @@ class Workflow(BaseWorkflow):
     """
     Error = WorkflowError
 
-    def __init__(self, workdir, runmode):
+    def __init__(self, workdir, manager):
         """
         Args:
             workdir:
                 Path to the working directory.
-            runmode:
-                RunMode instance or string "sequential"
+            manager:
+                `TaskAdapter` object.
         """
         self.workdir = os.path.abspath(workdir)
 
-        self.runmode = RunMode.asrunmode(runmode)
+        self.manager = manager
 
         self._tasks = []
 
@@ -377,9 +380,21 @@ class Workflow(BaseWorkflow):
         return [task.process for task in self]
 
     @property
-    def alldone(self):
-        """True if all the Task in the `Workflow` are completed."""
-        return all([task.status == Task.S_DONE for task in self])
+    def all_done(self):
+        """True if all the Task in the `Workflow` are done."""
+        return all([task.status >= Task.S_DONE for task in self])
+
+    def status_counter(self):
+        """
+        Returns a `Counter` object that counts the number of task with 
+        given status (use the string representation of the status as key).
+        """
+        counter = collections.Counter() 
+
+        for task in self:
+            counter[task.str_status] += 1
+
+        return counter
 
     @property
     def isnc(self):
@@ -395,7 +410,6 @@ class Workflow(BaseWorkflow):
     #def to_dict(self):
     #    d = dict(
     #        workdir=self.workdir,
-    #        runmode=self.runmode.to_dict,
     #        kwargs=self._kwargs
     #    )
     #    d["@module"] = self.__class__.__module__
@@ -404,7 +418,7 @@ class Workflow(BaseWorkflow):
 
     #@classmethod
     #def from_dict(cls, d):
-    #    return cls(d["workdir"], d["runmode"])
+    #    return cls(d["workdir"])
 
     def register(self, obj, links=()):
         """
@@ -422,20 +436,16 @@ class Workflow(BaseWorkflow):
         if links and not isinstance(links, collections.Iterable):
             links = [links,]
 
-        task_id = len(self) + 1
+        task_id = len(self)
         task_workdir = os.path.join(self.workdir, "task_" + str(task_id))
 
         if isinstance(obj, Strategy):
             # Create the new task (note the factory so that we create subclasses easily).
-            task = task_factory(obj, task_workdir, self.runmode, task_id=task_id, links=links)
+            task = task_factory(obj, task_workdir, self.manager, task_id=task_id, links=links)
 
         else:
             # Create the new task from the input. Note that no subclasses are instanciated here.
-            try:
-                task = AbinitTask.from_input(obj, task_workdir, self.runmode, task_id=task_id, links=links)
-
-            except:
-                raise TypeError("Don't know how to create a Task from object type %s" % str(type(obj)))
+            task = AbinitTask.from_input(obj, task_workdir, self.manager, task_id=task_id, links=links)
 
         self._tasks.append(task)
 
@@ -446,7 +456,7 @@ class Workflow(BaseWorkflow):
         return WorkLink(task)
 
     def path_in_workdir(self, filename):
-        """Create the absolute path of filename in the workind directory."""
+        """Create the absolute path of filename in the working directory."""
         return os.path.join(self.workdir, filename)
 
     def setup(self, *args, **kwargs):
@@ -581,7 +591,7 @@ class Workflow(BaseWorkflow):
         Return a numpy array with the total energies in Hartree
         The array element is set to np.inf if an exception is raised while reading the GSR file.
         """
-        if not self.alldone:
+        if not self.all_done:
             raise self.Error("Some task is still in running/submitted state")
 
         etotal = []
@@ -607,20 +617,20 @@ class IterativeWork(Workflow):
     """
     __metaclass__ = abc.ABCMeta
 
-    def __init__(self, workdir, runmode, strategy_generator, max_niter=25):
+    def __init__(self, workdir, strategy_generator, manager, max_niter=25):
         """
         Args:
             workdir:
                 Working directory.
-            runmode:
-                `RunMode` object.
             strategy_generator:
                 Strategy generator.
+            manager:
+                `TaskManager` class.
             max_niter:
                 Maximum number of iterations. A negative value or zero value
                 is equivalent to having an infinite number of iterations.
         """
-        super(IterativeWork, self).__init__(workdir, runmode)
+        super(IterativeWork, self).__init__(workdir, manager)
 
         self.strategy_generator = strategy_generator
 
@@ -635,6 +645,7 @@ class IterativeWork(Workflow):
         """
         try:
             next_strategy = next(self.strategy_generator)
+
         except StopIteration:
             raise StopIteration
 
@@ -916,14 +927,14 @@ def plot_etotal(ecut_list, etotals, aug_ratios, **kwargs):
 
 class PseudoConvergence(Workflow):
 
-    def __init__(self, workdir, pseudo, ecut_list, atols_mev,
-                 runmode="sequential", toldfe=1.e-8, spin_mode="polarized", 
-                 acell=(8, 9, 10), smearing="fermi_dirac:0.1 eV",):
+    def __init__(self, workdir, manager, pseudo, ecut_list, atols_mev,
+                 toldfe=1.e-8, spin_mode="polarized", 
+                 acell=(8, 9, 10), smearing="fermi_dirac:0.1 eV"):
 
-        super(PseudoConvergence, self).__init__(workdir, runmode)
+        super(PseudoConvergence, self).__init__(workdir, manager)
 
         # Temporary object used to build the strategy.
-        generator = PseudoIterativeConvergence(workdir, pseudo, ecut_list, atols_mev,
+        generator = PseudoIterativeConvergence(workdir, manager, pseudo, ecut_list, atols_mev,
                                                toldfe    = toldfe,
                                                spin_mode = spin_mode,
                                                acell     = acell,
@@ -964,8 +975,8 @@ class PseudoConvergence(Workflow):
 
 class PseudoIterativeConvergence(IterativeWork):
 
-    def __init__(self, workdir, pseudo, ecut_list_or_slice, atols_mev,
-                 runmode="sequential", toldfe=1.e-8, spin_mode="polarized", 
+    def __init__(self, workdir, manager, pseudo, ecut_list_or_slice, atols_mev,
+                 toldfe=1.e-8, spin_mode="polarized", 
                  acell=(8, 9, 10), smearing="fermi_dirac:0.1 eV", max_niter=50,):
         """
         Args:
@@ -977,6 +988,8 @@ class PseudoIterativeConvergence(IterativeWork):
                 List of cutoff energies or slice object (mainly used for infinite iterations).
             atols_mev:
                 List of absolute tolerances in meV (3 entries corresponding to accuracy ["low", "normal", "high"]
+            manager:
+                `TaskManager` object.
             spin_mode:
                 Defined how the electronic spin will be treated.
             acell:
@@ -1003,7 +1016,7 @@ class PseudoIterativeConvergence(IterativeWork):
                 yield self.strategy_with_ecut(ecut)
 
         super(PseudoIterativeConvergence, self).__init__(
-            workdir, runmode, strategy_generator(), max_niter=max_niter)
+            workdir, manager, strategy_generator(), max_niter=max_niter)
 
         if not self.isnc:
             raise NotImplementedError("PAW convergence tests are not supported yet")
@@ -1071,14 +1084,13 @@ class PseudoIterativeConvergence(IterativeWork):
 
 class BandStructure(Workflow):
     """Workflow for band structure calculations."""
-    def __init__(self, workdir, runmode, scf_strategy, nscf_strategy,
-                 dos_strategy=None):
+    def __init__(self, workdir, manager, scf_strategy, nscf_strategy, dos_strategy=None):
         """
         Args:
             workdir:
                 Working directory.
-            runmode:
-                `RunMode` instance.
+            manager:
+                `TaskManager` object.
             scf_strategy:
                 `SCFStrategy` instance
             nscf_strategy:
@@ -1087,7 +1099,7 @@ class BandStructure(Workflow):
                 `NSCFStrategy` instance defining the DOS calculation. 
                 DOS is computed only if dos_strategy is not None.
         """
-        super(BandStructure, self).__init__(workdir, runmode)
+        super(BandStructure, self).__init__(workdir, manager)
 
         # Register the GS-SCF run.
         scf_link = self.register(scf_strategy)
@@ -1103,17 +1115,17 @@ class BandStructure(Workflow):
 
 class Relaxation(Workflow):
 
-    def __init__(self, workdir, runmode, relax_strategy):
+    def __init__(self, workdir, manager, relax_strategy):
         """
         Args:
             workdir:
                 Working directory.
-            runmode:
-                `RunMode` instance.
+            manager:
+                `TaskManager` object.
             relax_strategy:
                 `RelaxStrategy` instance
         """
-        super(Relaxation, self).__init__(workdir, runmode)
+        super(Relaxation, self).__init__(workdir, manager)
 
         link = self.register(relax_strategy)
 
@@ -1121,11 +1133,11 @@ class Relaxation(Workflow):
 
 class DeltaTest(Workflow):
 
-    def __init__(self, workdir, runmode, structure_or_cif, pseudos, kppa,
+    def __init__(self, workdir, manager, structure_or_cif, pseudos, kppa,
                  spin_mode="polarized", toldfe=1.e-8, smearing="fermi_dirac:0.1 eV",
                  accuracy="normal", ecut=None, ecutsm=0.05, chksymbreak=0): # FIXME Hack
 
-        super(DeltaTest, self).__init__(workdir, runmode)
+        super(DeltaTest, self).__init__(workdir, manager)
 
         if isinstance(structure_or_cif, Structure):
             structure = structure_or_cif
@@ -1221,7 +1233,7 @@ class DeltaTest(Workflow):
 
 class GW_Workflow(Workflow):
 
-    def __init__(self, workdir, runmode, scf_strategy, nscf_strategy,
+    def __init__(self, workdir, manager, scf_strategy, nscf_strategy,
                  scr_strategy, sigma_strategy):
         """
         Workflow for GW calculations.
@@ -1229,8 +1241,8 @@ class GW_Workflow(Workflow):
         Args:
             workdir:
                 Working directory of the calculation.
-            runmode:
-                `RunMode` instance.
+            manager:
+                `TaskManager` object.
             scf_strategy:
                 `SCFStrategy` instance
             nscf_strategy:
@@ -1240,7 +1252,7 @@ class GW_Workflow(Workflow):
             sigma_strategy:
                 Strategy for the self-energy run.
         """
-        super(GW_Workflow, self).__init__(workdir, runmode)
+        super(GW_Workflow, self).__init__(workdir, manager)
 
         # Register the GS-SCF run.
         scf_link = self.register(scf_strategy)
@@ -1258,7 +1270,7 @@ class GW_Workflow(Workflow):
 
 class BSEMDF_Workflow(Workflow):
 
-    def __init__(self, workdir, runmode, scf_strategy, nscf_strategy, bse_strategy): 
+    def __init__(self, workdir, manager, scf_strategy, nscf_strategy, bse_strategy):
         """
         Workflow for simple BSE calculations in which the self-energy corrections 
         are approximated by the scissors operator and the screening in modeled 
@@ -1267,8 +1279,8 @@ class BSEMDF_Workflow(Workflow):
         Args:
             workdir:
                 Working directory of the calculation.
-            runmode:
-                Run mode.
+            manager:
+                `TaskManager`.
             scf_strategy:
                 ScfStrategy instance
             nscf_strategy:
@@ -1276,7 +1288,7 @@ class BSEMDF_Workflow(Workflow):
             bse_strategy:
                 BSEStrategy instance.
         """
-        super(BSEMDF_Workflow, self).__init__(workdir, runmode)
+        super(BSEMDF_Workflow, self).__init__(workdir, manager)
 
         # Register the GS-SCF run.
         scf_link = self.register(scf_strategy)
